@@ -5,5 +5,6 @@ on specific data.
 """
 
 from .subscribe import subscribe, create_source_event_stream
+from .map_async_iterator import MapAsyncIterator
 
-__all__ = ["subscribe", "create_source_event_stream"]
+__all__ = ["subscribe", "create_source_event_stream", "MapAsyncIterator"]
