@@ -406,14 +406,14 @@ class GraphQLScalarType(GraphQLNamedType):
         return value
 
     def parse_literal(
-        self, node: ValueNode, _variables: Optional[Dict[str, Any]] = None
+        self, node: ValueNode, variables: Optional[Dict[str, Any]] = None
     ) -> Any:
         """Parses an externally provided literal value to use as an input.
 
         This default method uses the parse_value method and should be replaced
         with a more specific version when creating a scalar type.
         """
-        return self.parse_value(value_from_ast_untyped(node))
+        return self.parse_value(value_from_ast_untyped(node, variables))
 
     def to_kwargs(self) -> Dict[str, Any]:
         return dict(
@@ -550,6 +550,7 @@ class GraphQLField:
 
     @property
     def is_deprecated(self) -> bool:
+        # this property is officially deprecated, but we still keep it here
         return self.deprecation_reason is not None
 
 
@@ -587,6 +588,7 @@ GraphQLFieldResolver = Callable[..., Any]
 
 # Note: Contrary to the Javascript implementation of GraphQLTypeResolver,
 # the context is passed as part of the GraphQLResolveInfo:
+# Note: returning GraphQLObjectType is deprecated and will be removed.
 GraphQLTypeResolver = Callable[
     [Any, GraphQLResolveInfo, "GraphQLAbstractType"],
     AwaitableOrValue[Optional[Union["GraphQLObjectType", str]]],
@@ -603,6 +605,7 @@ class GraphQLArgument:
     type: "GraphQLInputType"
     default_value: Any
     description: Optional[str]
+    deprecation_reason: Optional[str]
     out_name: Optional[str]  # for transforming names (extension of GraphQL.js)
     extensions: Optional[Dict[str, Any]]
     ast_node: Optional[InputValueDefinitionNode]
@@ -612,6 +615,7 @@ class GraphQLArgument:
         type_: "GraphQLInputType",
         default_value: Any = Undefined,
         description: Optional[str] = None,
+        deprecation_reason: Optional[str] = None,
         out_name: Optional[str] = None,
         extensions: Optional[Dict[str, Any]] = None,
         ast_node: Optional[InputValueDefinitionNode] = None,
@@ -620,6 +624,8 @@ class GraphQLArgument:
             raise TypeError("Argument type must be a GraphQL input type.")
         if description is not None and not is_description(description):
             raise TypeError("Argument description must be a string.")
+        if deprecation_reason is not None and not is_description(deprecation_reason):
+            raise TypeError("Argument deprecation reason must be a string.")
         if out_name is not None and not isinstance(out_name, str):
             raise TypeError("Argument out name must be a string.")
         if extensions is not None and (
@@ -634,6 +640,7 @@ class GraphQLArgument:
         self.type = type_
         self.default_value = default_value
         self.description = description
+        self.deprecation_reason = deprecation_reason
         self.out_name = out_name
         self.extensions = extensions
         self.ast_node = ast_node
@@ -644,6 +651,7 @@ class GraphQLArgument:
             and self.type == other.type
             and self.default_value == other.default_value
             and self.description == other.description
+            and self.deprecation_reason == other.deprecation_reason
             and self.out_name == other.out_name
             and self.extensions == other.extensions
         )
@@ -653,6 +661,7 @@ class GraphQLArgument:
             type_=self.type,
             default_value=self.default_value,
             description=self.description,
+            deprecation_reason=self.deprecation_reason,
             out_name=self.out_name,
             extensions=self.extensions,
             ast_node=self.ast_node,
@@ -1261,6 +1270,7 @@ class GraphQLEnumValue:
 
     @property
     def is_deprecated(self) -> bool:
+        # this property is officially deprecated, but we still keep it here
         return self.deprecation_reason is not None
 
 
@@ -1397,6 +1407,7 @@ class GraphQLInputField:
     type: "GraphQLInputType"
     default_value: Any
     description: Optional[str]
+    deprecation_reason: Optional[str]
     out_name: Optional[str]  # for transforming names (extension of GraphQL.js)
     extensions: Optional[Dict[str, Any]]
     ast_node: Optional[InputValueDefinitionNode]
@@ -1406,6 +1417,7 @@ class GraphQLInputField:
         type_: "GraphQLInputType",
         default_value: Any = Undefined,
         description: Optional[str] = None,
+        deprecation_reason: Optional[str] = None,
         out_name: Optional[str] = None,
         extensions: Optional[Dict[str, Any]] = None,
         ast_node: Optional[InputValueDefinitionNode] = None,
@@ -1414,6 +1426,8 @@ class GraphQLInputField:
             raise TypeError("Input field type must be a GraphQL input type.")
         if description is not None and not is_description(description):
             raise TypeError("Input field description must be a string.")
+        if deprecation_reason is not None and not is_description(deprecation_reason):
+            raise TypeError("Input field deprecation reason must be a string.")
         if out_name is not None and not isinstance(out_name, str):
             raise TypeError("Input field out name must be a string.")
         if extensions is not None and (
@@ -1428,6 +1442,7 @@ class GraphQLInputField:
         self.type = type_
         self.default_value = default_value
         self.description = description
+        self.deprecation_reason = deprecation_reason
         self.out_name = out_name
         self.extensions = extensions
         self.ast_node = ast_node
@@ -1438,6 +1453,7 @@ class GraphQLInputField:
             and self.type == other.type
             and self.default_value == other.default_value
             and self.description == other.description
+            and self.deprecation_reason == other.deprecation_reason
             and self.extensions == other.extensions
             and self.out_name == other.out_name
         )
@@ -1447,6 +1463,7 @@ class GraphQLInputField:
             type_=self.type,
             default_value=self.default_value,
             description=self.description,
+            deprecation_reason=self.deprecation_reason,
             out_name=self.out_name,
             extensions=self.extensions,
             ast_node=self.ast_node,
