@@ -1,3 +1,4 @@
+from copy import copy, deepcopy
 from typing import (
     Any,
     Collection,
@@ -21,11 +22,13 @@ from .definition import (
     GraphQLObjectType,
     GraphQLUnionType,
     GraphQLType,
+    GraphQLWrappingType,
     get_named_type,
     is_input_object_type,
     is_interface_type,
     is_object_type,
     is_union_type,
+    is_wrapping_type,
 )
 from .directives import GraphQLDirective, specified_directives, is_directive
 from .introspection import introspection_types
@@ -285,6 +288,43 @@ class GraphQLSchema:
             assume_valid=self._validation_errors is not None,
         )
 
+    def __copy__(self) -> "GraphQLSchema":  # pragma: no cover
+        return self.__class__(**self.to_kwargs())
+
+    def __deepcopy__(self, memo_: Dict) -> "GraphQLSchema":
+        from ..type import (
+            is_introspection_type,
+            is_specified_scalar_type,
+            is_specified_directive,
+        )
+
+        type_map: TypeMap = {
+            name: copy(type_)
+            for name, type_ in self.type_map.items()
+            if not is_introspection_type(type_) and not is_specified_scalar_type(type_)
+        }
+        types = type_map.values()
+        for type_ in types:
+            remap_named_type(type_, type_map)
+        directives = [
+            directive if is_specified_directive(directive) else copy(directive)
+            for directive in self.directives
+        ]
+        return self.__class__(
+            self.query_type and cast(GraphQLObjectType, type_map[self.query_type.name]),
+            self.mutation_type
+            and cast(GraphQLObjectType, type_map[self.mutation_type.name]),
+            self.subscription_type
+            and cast(GraphQLObjectType, type_map[self.subscription_type.name]),
+            types,
+            directives,
+            self.description,
+            extensions=deepcopy(self.extensions),
+            ast_node=deepcopy(self.ast_node),
+            extension_ast_nodes=deepcopy(self.extension_ast_nodes),
+            assume_valid=True,
+        )
+
     def get_type(self, name: str) -> Optional[GraphQLNamedType]:
         return self.type_map.get(name)
 
@@ -317,7 +357,9 @@ class GraphQLSchema:
         return self.is_sub_type(abstract_type, possible_type)
 
     def is_sub_type(
-        self, abstract_type: GraphQLAbstractType, maybe_sub_type: GraphQLNamedType,
+        self,
+        abstract_type: GraphQLAbstractType,
+        maybe_sub_type: GraphQLNamedType,
     ) -> bool:
         """Check whether a type is a subtype of a given abstract type."""
         types = self._sub_type_map.get(abstract_type.name)
@@ -396,3 +438,44 @@ def assert_schema(schema: Any) -> GraphQLSchema:
     if not is_schema(schema):
         raise TypeError(f"Expected {inspect(schema)} to be a GraphQL schema.")
     return cast(GraphQLSchema, schema)
+
+
+def remapped_type(type_: GraphQLType, type_map: TypeMap) -> GraphQLType:
+    """Get a copy of the given type that uses this type map."""
+    if is_wrapping_type(type_):
+        type_ = cast(GraphQLWrappingType, type_)
+        return type_.__class__(remapped_type(type_.of_type, type_map))
+    type_ = cast(GraphQLNamedType, type_)
+    return type_map.get(type_.name, type_)
+
+
+def remap_named_type(type_: GraphQLNamedType, type_map: TypeMap) -> None:
+    """Change all references in the given named type to use this type map."""
+    if is_union_type(type_):
+        type_ = cast(GraphQLUnionType, type_)
+        type_.types = [
+            type_map.get(member_type.name, member_type) for member_type in type_.types
+        ]
+    elif is_object_type(type_) or is_interface_type(type_):
+        type_ = cast(Union[GraphQLObjectType, GraphQLInterfaceType], type_)
+        type_.interfaces = [
+            type_map.get(interface_type.name, interface_type)
+            for interface_type in type_.interfaces
+        ]
+        fields = type_.fields
+        for field_name, field in fields.items():
+            field = copy(field)
+            field.type = remapped_type(field.type, type_map)
+            args = field.args
+            for arg_name, arg in args.items():
+                arg = copy(arg)
+                arg.type = remapped_type(arg.type, type_map)
+                args[arg_name] = arg
+            fields[field_name] = field
+    elif is_input_object_type(type_):
+        type_ = cast(GraphQLInputObjectType, type_)
+        fields = type_.fields
+        for field_name, field in fields.items():
+            field = copy(field)
+            field.type = remapped_type(field.type, type_map)
+            fields[field_name] = field
